/**
* Filename: RmsSelector.java (in org.openbandy.io.rms)
* This is part of the OpenBandy project.
* 
* www.openbandy.org
*/

package org.openbandy.io.rms;

import java.util.Enumeration;
import java.util.Vector;

import org.openbandy.io.Serializable;
import org.openbandy.io.SerializableFactory;
import org.openbandy.io.storage.Selector;


/**
* The RmsSelector is an implementation of the <code>Selector</code> interface
* for the RecordManagementStore (RMS).
*
* <br><br>
* (c) Copyright P. Bolliger 2007, ALL RIGHTS RESERVED.
*
* @author Philipp Bolliger (philipp@bolliger.name)
* @version 1.0
*/
public class RmsSelector implements Selector {

	/* The database serializer */
	private RmsSerializer rmsSerializer;

	/* Constructor */ 
	public RmsSelector(RmsSerializer rmsSerializer){
		this.rmsSerializer = rmsSerializer;
	}


	/* *************** Methods for Selector ***************** */

	/* (non-Javadoc)
	 * @see org.openbandy.io.storage.Selector#getAll(org.openbandy.io.Serializable)
	 */
	public Vector getAll(Serializable serializable) {
		/* vector to return */
		Vector containedObjects = new Vector();
		
		if(serializable != null){
			/* resolve type record store name and get corresponding connection */
			String typeName = RmsStorage.getTypeRecordStoreName(serializable);        
			RmsConnection typeConn = RmsStorage.getConnection(typeName);

			if(typeConn != null){

				/* connect to record store */
				RmsRecord types = new RmsRecord(typeConn.getRecordStore());

				/* read header */
				types.moveTo(1);
				String headerString = types.readUTF(); 
				if(RmsTypeRecordStoreHeader.isValidHeaderString(headerString)){
					RmsTypeRecordStoreHeader typesStoreHeader = new RmsTypeRecordStoreHeader();
					typesStoreHeader.setHeader(headerString);

					/* iterate over all entries */
					Enumeration elements = typesStoreHeader.getTypeRecordStoreIds();
					while(elements.hasMoreElements()){
						String typeIdString = (String)elements.nextElement();
						int typeId = Integer.parseInt(typeIdString);
						types.moveTo(typeId);

						/* read name of object store and get corresponding connection */
						String objectStoreName = types.readUTF();      
						RmsConnection objectConn = RmsStorage.getConnection(objectStoreName);

						if(objectConn != null){
							/* deserialize object */
							Serializable object = SerializableFactory.getNewSerializableObject(serializable.getClass().getName());      
							rmsSerializer.deserializeObject(object, objectConn);

							/* add to vector */
							containedObjects.addElement(object);
						}
					}
				}
			}
		}
		return containedObjects;
	}


	/* (non-Javadoc)
	 * @see org.openbandy.io.storage.Selector#getObjectsWhere(java.lang.String, java.lang.String, java.lang.String)
	 */
	public Vector getObjectsWhere(String typename, String attributeName, String attributeValue) {
		// TODO Auto-generated method stub
		return null;
	}    
}
