package org.openbandy.pref;

import java.util.Enumeration;

import javax.microedition.lcdui.Command;
import javax.microedition.lcdui.CommandListener;
import javax.microedition.lcdui.Display;
import javax.microedition.lcdui.Displayable;
import javax.microedition.lcdui.Form;
import javax.microedition.lcdui.TextField;

import org.openbandy.service.LogService;
import org.openbandy.ui.BandyDisplayable;


public class PreferencesForm extends Form implements BandyDisplayable, CommandListener{

	/** Reference to the MIDlets display */
	protected Display display;

	/** Reference to the MIDlets previous displayable */
	protected Displayable previousDisplayable;

	/** Maximum size of preference value (in text field) */
	private static final int MAX_SIZE = 20;

	/* Commands */
	private Command cmdSave;
	private Command cmdCancel;
	private Command cmdClear;

	/* Constructor */
	public PreferencesForm(){
		super("Preferences");

		/* add commands */
		cmdSave = new Command("Ok", Command.OK, 1);
		cmdCancel = new Command("Back", Command.CANCEL, 1);
		cmdClear = new Command("Clear Fields", Command.SCREEN, 2);
		addCommand(cmdSave);
		addCommand(cmdCancel);
		addCommand(cmdClear);

		/* make myself command listener */
		this.setCommandListener(this);
	}


	/* ******************* Methods for BandyDisplayable ******** */

	public void setDisplay(Display display, Displayable previousDisplayable) {
		this.display = display;
		this.previousDisplayable = previousDisplayable;
	}

	public void updateScreen() {

		/* remove all text fields from the screen and the list */
		this.deleteAll();

		/* create text fields and add them to the array and the screen */
		for(Enumeration preferences = Preferences.preferences(); preferences.hasMoreElements();) {
			try {
				Preference preference = (Preference)preferences.nextElement();
				if(preference.isScreenModifiable) {
					/* create new text fiel */
					TextField textField = new TextField(preference.name, preference.value, MAX_SIZE, TextField.ANY);

					/* add text field to screen */
					append(textField);
				}
			}
			catch (Exception e) {
				LogService.error(this, e.getMessage(), e);
			}
		}
	}

	/* ******************* Methods for CommandListener ******** */

	public void commandAction(Command c, Displayable d) {
		if(c == cmdCancel) {
			display.setCurrent(previousDisplayable);
		}
		else if(c == cmdSave) {			
			for(int i=0; i<this.size();i++){
				try {
					TextField textField = (TextField)get(i);
					Preferences.setValue(textField.getLabel(), textField.getString());
				}
				catch(ClassCastException cce) {
					/* do nothing, just ignore */
				}
				catch (Exception e) {
					LogService.error(this, e.getMessage(), e);
				}	
			}

			/* go back to the previous screen */
			display.setCurrent(previousDisplayable);
		}
		else if(c == cmdClear) {
			for(int i=0; i<this.size();i++){
				try {
					TextField textField = (TextField)get(i);
					Preferences.setValue(textField.getLabel(), "");
				}
				catch(ClassCastException cce) {
					/* do nothing, just ignore */
				}
				catch (Exception e) {
					LogService.error(this, e.getMessage(), e);
				}
			}
		}
	}

}
